@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
+# Copyright (c) 2016-2021 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -27,14 +27,17 @@
 
 __authors__ = ["H. Payno", "T. Vincent"]
 __license__ = "MIT"
-__date__ = "11/04/2017"
+__date__ = "24/04/2018"
 
 
 import logging
+import weakref
 import numpy
+
 from ._utils import ticklayout
-from .. import qt, icons
-from silx.gui.plot import Colormap
+from .. import qt
+from ..qt import inspect as qt_inspect
+from silx.gui import colors
 
 _logger = logging.getLogger(__name__)
 
@@ -65,11 +68,12 @@ class ColorBarWidget(qt.QWidget):
     :param plot: PlotWidget the colorbar is attached to (optional)
     :param str legend: the label to set to the colorbar
     """
+    sigVisibleChanged = qt.Signal(bool)
+    """Emitted when the property `visible` have changed."""
 
     def __init__(self, parent=None, plot=None, legend=None):
         self._isConnected = False
-        self._plot = None
-        self._viewAction = None
+        self._plotRef = None
         self._colormap = None
         self._data = None
 
@@ -95,7 +99,7 @@ class ColorBarWidget(qt.QWidget):
 
     def getPlot(self):
         """Returns the :class:`Plot` associated to this widget or None"""
-        return self._plot
+        return None if self._plotRef is None else self._plotRef()
 
     def setPlot(self, plot):
         """Associate a plot to the ColorBar
@@ -104,55 +108,65 @@ class ColorBarWidget(qt.QWidget):
                      If None will remove any connection with a previous plot.
         """
         self._disconnectPlot()
-        self._plot = plot
+        self._plotRef = None if plot is None else weakref.ref(plot)
         self._connectPlot()
 
     def _disconnectPlot(self):
         """Disconnect from Plot signals"""
-        if self._plot is not None and self._isConnected:
+        if self._isConnected:
             self._isConnected = False
-            self._plot.sigActiveImageChanged.disconnect(
-                self._activeImageChanged)
-            self._plot.sigPlotSignal.disconnect(self._defaultColormapChanged)
+            plot = self.getPlot()
+            if plot is not None and qt_inspect.isValid(plot):
+                plot.sigActiveImageChanged.disconnect(
+                    self._activeImageChanged)
+                plot.sigActiveScatterChanged.disconnect(
+                    self._activeScatterChanged)
+                plot.sigPlotSignal.disconnect(self._defaultColormapChanged)
 
     def _connectPlot(self):
         """Connect to Plot signals"""
-        if self._plot is not None and not self._isConnected:
-            activeImageLegend = self._plot.getActiveImage(just_legend=True)
-            if activeImageLegend is None:  # Show plot default colormap
+        plot = self.getPlot()
+        if plot is not None and not self._isConnected:
+            activeImageLegend = plot.getActiveImage(just_legend=True)
+            activeScatterLegend = plot._getActiveItem(
+                kind='scatter', just_legend=True)
+            if activeImageLegend is None and activeScatterLegend is None:
+                # Show plot default colormap
                 self._syncWithDefaultColormap()
-            else:  # Show active image colormap
+            elif activeImageLegend is not None:  # Show active image colormap
                 self._activeImageChanged(None, activeImageLegend)
-            self._plot.sigActiveImageChanged.connect(self._activeImageChanged)
-            self._plot.sigPlotSignal.connect(self._defaultColormapChanged)
+            elif activeScatterLegend is not None:  # Show active scatter colormap
+                self._activeScatterChanged(None, activeScatterLegend)
+
+            plot.sigActiveImageChanged.connect(self._activeImageChanged)
+            plot.sigActiveScatterChanged.connect(self._activeScatterChanged)
+            plot.sigPlotSignal.connect(self._defaultColormapChanged)
             self._isConnected = True
 
+    def setVisible(self, isVisible):
+        qt.QWidget.setVisible(self, isVisible)
+        self.sigVisibleChanged.emit(isVisible)
+
     def showEvent(self, event):
         self._connectPlot()
-        if self._viewAction is not None:
-            self._viewAction.setChecked(True)
 
     def hideEvent(self, event):
         self._disconnectPlot()
-        if self._viewAction is not None:
-            self._viewAction.setChecked(False)
 
     def getColormap(self):
-        """
-
-        :return: the :class:`.Colormap` colormap displayed in the colorbar.
+        """Returns the colormap displayed in the colorbar.
 
+        :rtype: ~silx.gui.colors.Colormap
         """
         return self.getColorScaleBar().getColormap()
 
     def setColormap(self, colormap, data=None):
         """Set the colormap to be displayed.
 
-        :param colormap: The colormap to apply on the
-            ColorBarWidget
-        :type colormap: :class:`.Colormap`
-        :param numpy.ndarray data: the data to display, needed if the colormap
-            require an autoscale
+        :param ~silx.gui.colors.Colormap colormap:
+            The colormap to apply on the ColorBarWidget
+        :param Union[numpy.ndarray,~silx.gui.plot.items.ColormapMixin] data:
+            The data to display or item, needed if the colormap require an autoscale
         """
         self._data = data
         self.getColorScaleBar().setColormap(colormap=colormap,
@@ -190,38 +204,63 @@ class ColorBarWidget(qt.QWidget):
         :return: return the legend displayed along the colorbar
         :rtype: str
         """
-        return self.legend.getText()
+        return self.legend.text()
+
+    def _activeScatterChanged(self, previous, legend):
+        """Handle plot active scatter changed"""
+        plot = self.getPlot()
+
+        # Do not handle active scatter while there is an image
+        if plot.getActiveImage() is not None:
+            return
+
+        if legend is None:  # No active scatter, display no colormap
+            self.setColormap(colormap=None)
+            return
+
+        # Sync with active scatter
+        scatter = plot._getActiveItem(kind='scatter')
+
+        self.setColormap(colormap=scatter.getColormap(),
+                         data=scatter)
 
     def _activeImageChanged(self, previous, legend):
-        """Handle plot active curve changed"""
-        if legend is None:  # No active image, display no colormap
-            self.setColormap(colormap=None)
-            return
+        """Handle plot active image changed"""
+        plot = self.getPlot()
 
-        # Sync with active image
-        image = self._plot.getActiveImage().getData(copy=False)
+        if legend is None:  # No active image, try with active scatter
+            activeScatterLegend = plot._getActiveItem(
+                kind='scatter', just_legend=True)
+            # No more active image, use active scatter if any
+            self._activeScatterChanged(None, activeScatterLegend)
+        else:
+            # Sync with active image
+            image = plot.getActiveImage()
 
-        # RGB(A) image, display default colormap
-        if image.ndim != 2:
-            self.setColormap(colormap=None)
-            return
+            # RGB(A) image, display default colormap
+            array = image.getData(copy=False)
+            if array.ndim != 2:
+                self.setColormap(colormap=None)
+                return
 
-        # data image, sync with image colormap
-        # do we need the copy here : used in the case we are changing
-        # vmin and vmax but should have already be done by the plot
-        self.setColormap(colormap=self._plot.getActiveImage().getColormap(),
-                         data=image)
+            # data image, sync with image colormap
+            # do we need the copy here : used in the case we are changing
+            # vmin and vmax but should have already be done by the plot
+            self.setColormap(colormap=image.getColormap(), data=image)
 
     def _defaultColormapChanged(self, event):
         """Handle plot default colormap changed"""
-        if (event['event'] == 'defaultColormapChanged' and
-                self._plot.getActiveImage() is None):
-            # No active image, take default colormap update into account
-            self._syncWithDefaultColormap()
+        if event['event'] == 'defaultColormapChanged':
+            plot = self.getPlot()
+            if (plot is not None and
+                    plot.getActiveImage() is None and
+                    plot._getActiveItem(kind='scatter') is None):
+                # No active item, take default colormap update into account
+                self._syncWithDefaultColormap()
 
-    def _syncWithDefaultColormap(self, data=None):
+    def _syncWithDefaultColormap(self):
         """Update colorbar according to plot default colormap"""
-        self.setColormap(self._plot.getDefaultColormap(), data)
+        self.setColormap(self.getPlot().getDefaultColormap())
 
     def getColorScaleBar(self):
         """
@@ -230,21 +269,6 @@ class ColorBarWidget(qt.QWidget):
             and ticks"""
         return self._colorScale
 
-    def getToggleViewAction(self):
-        """Returns a checkable action controlling this widget's visibility.
-
-        :rtype: QAction
-        """
-        if self._viewAction is None:
-            self._viewAction = qt.QAction(self)
-            self._viewAction.setText('Colorbar')
-            self._viewAction.setIcon(icons.getQIcon('colorbar'))
-            self._viewAction.setToolTip('Show/Hide the colorbar')
-            self._viewAction.setCheckable(True)
-            self._viewAction.setChecked(self.isVisible())
-            self._viewAction.toggled[bool].connect(self.setVisible)
-        return self._viewAction
-
 
 class _VerticalLegend(qt.QLabel):
     """Display vertically the given text
@@ -326,13 +350,14 @@ class ColorScaleBar(qt.QWidget):
                                       margin=ColorScaleBar._TEXT_MARGIN)
         if colormap:
             vmin, vmax = colormap.getColormapRange(data)
+            normalizer = colormap._getNormalizer()
         else:
-            vmin, vmax = Colormap.DEFAULT_MIN_LIN, Colormap.DEFAULT_MAX_LIN
+            vmin, vmax = colors.DEFAULT_MIN_LIN, colors.DEFAULT_MAX_LIN
+            normalizer = None
 
-        norm = colormap.getNormalization() if colormap else Colormap.Colormap.LINEAR
         self.tickbar = _TickBar(vmin=vmin,
                                 vmax=vmax,
-                                norm=norm,
+                                normalizer=normalizer,
                                 parent=self,
                                 displayValues=displayTicksValues,
                                 margin=ColorScaleBar._TEXT_MARGIN)
@@ -383,21 +408,21 @@ class ColorScaleBar(qt.QWidget):
         """Set the new colormap to be displayed
 
         :param Colormap colormap: the colormap to set
-        :param numpy.ndarray data: the data to display, needed if the colormap
-            require an autoscale
+        :param Union[numpy.ndarray,~silx.gui.plot.items.Item] data:
+            The data or item to display, needed if the colormap requires an autoscale
         """
         self.colorScale.setColormap(colormap, data)
 
         if colormap is not None:
             vmin, vmax = colormap.getColormapRange(data)
-            norm = colormap.getNormalization()
+            normalizer = colormap._getNormalizer()
         else:
             vmin, vmax = None, None
-            norm = None
+            normalizer = None
 
         self.tickbar.update(vmin=vmin,
                             vmax=vmax,
-                            norm=norm)
+                            normalizer=normalizer)
         self._setMinMaxLabels(vmin, vmax)
 
     def setMinMaxVisible(self, val=True):
@@ -405,8 +430,8 @@ class ColorScaleBar(qt.QWidget):
 
         :param val: if True, set the labels visible, otherwise set it not visible
         """
-        self._maxLabel.show() if val is True else self._maxLabel.hide()
-        self._minLabel.show() if val is True else self._minLabel.hide()
+        self._minLabel.setVisible(val)
+        self._maxLabel.setVisible(val)
 
     def _updateMinMax(self):
         """Update the min and max label if we are in the case of the
@@ -478,6 +503,8 @@ class _ColorScale(qt.QWidget):
     :param colormap: the colormap to be displayed
     :param parent: the Qt parent if any
     :param int margin: the top and left margin to apply.
+    :param Union[None,numpy.ndarray,~silx.gui.plot.items.ColormapMixin] data:
+        The data or item to use for getting the range for autoscale colormap.
 
     .. warning:: Value drawing will be
         done at the center of ticks. So if no margin is done your values
@@ -506,7 +533,8 @@ class _ColorScale(qt.QWidget):
         """Set the new colormap to be displayed
 
         :param dict colormap: the colormap to set
-        :param data: Optional data for which to compute colormap range.
+        :param Union[None,numpy.ndarray,~silx.gui.plot.items.ColormapMixin] data:
+            Optional data for which to compute colormap range.
         """
         self._colormap = colormap
         self.setEnabled(colormap is not None)
@@ -514,7 +542,7 @@ class _ColorScale(qt.QWidget):
         if colormap is None:
             self.vmin, self.vmax = None, None
         else:
-            assert colormap.getNormalization() in Colormap.Colormap.NORMALIZATIONS
+            assert colormap.getNormalization() in colors.Colormap.NORMALIZATIONS
             self.vmin, self.vmax = self._colormap.getColormapRange(data=data)
         self._updateColorGradient()
         self.update()
@@ -533,12 +561,7 @@ class _ColorScale(qt.QWidget):
             return
 
         indices = numpy.linspace(0., 1., self._NB_CONTROL_POINTS)
-        colormapDisp = Colormap.Colormap(name=colormap.getName(),
-                                         normalization=Colormap.Colormap.LINEAR,
-                                         vmin=None,
-                                         vmax=None,
-                                         colors=colormap.getColormapLUT())
-        colors = colormapDisp.applyToData(indices)
+        colors = colormap.getNColors(nbColors=self._NB_CONTROL_POINTS)
         self._gradient = qt.QLinearGradient(0, 1, 0, 0)
         self._gradient.setCoordinateMode(qt.QGradient.StretchToDeviceMode)
         self._gradient.setStops(
@@ -560,8 +583,8 @@ class _ColorScale(qt.QWidget):
         painter.drawRect(qt.QRect(
             0,
             self.margin,
-            self.width() - 1.,
-            self.height() - 2. * self.margin - 1.))
+            self.width() - 1,
+            self.height() - 2 * self.margin - 1))
 
     def mouseMoveEvent(self, event):
         tooltip = str(self.getValueFromRelativePosition(
@@ -586,19 +609,12 @@ class _ColorScale(qt.QWidget):
         if colormap is None:
             return
 
-        value = max(0.0, value)
-        value = min(value, 1.0)
+        value = numpy.clip(value, 0., 1.)
+        normalizer = colormap._getNormalizer()
+        normMin, normMax = normalizer.apply([self.vmin, self.vmax], self.vmin, self.vmax)
 
-        vmin = self.vmin
-        vmax = self.vmax
-        if colormap.getNormalization() == Colormap.Colormap.LINEAR:
-            return vmin + (vmax - vmin) * value
-        elif colormap.getNormalization() == Colormap.Colormap.LOGARITHM:
-            rpos = (numpy.log10(vmax) - numpy.log10(vmin)) * value + numpy.log10(vmin)
-            return numpy.power(10., rpos)
-        else:
-            err = "normalization type (%s) is not managed by the _ColorScale Widget" % colormap['normalization']
-            raise ValueError(err)
+        return normalizer.revert(
+            normMin + (normMax - normMin) * value, self.vmin, self.vmax)
 
     def setMargin(self, margin):
         """Define the margin to fit with a TickBar object.
@@ -607,7 +623,7 @@ class _ColorScale(qt.QWidget):
 
         :param int margin: the margin to apply on the top and bottom.
         """
-        self.margin = margin
+        self.margin = int(margin)
         self.update()
 
 
@@ -625,8 +641,7 @@ class _TickBar(qt.QWidget):
 
     :param int vmin: smaller value of the range of values
     :param int vmax: higher value of the range of values
-    :param str norm: normalization type to be displayed. Valid values are
-        'linear' and 'log'
+    :param normalizer: Normalization object.
     :param parent: the Qt parent if any
     :param bool displayValues: if True display the values close to the tick,
         Otherwise only signal it by '-'
@@ -646,7 +661,7 @@ class _TickBar(qt.QWidget):
 
     DEFAULT_TICK_DENSITY = 0.015
 
-    def __init__(self, vmin, vmax, norm, parent=None, displayValues=True,
+    def __init__(self, vmin, vmax, normalizer, parent=None, displayValues=True,
                  nticks=None, margin=5):
         super(_TickBar, self).__init__(parent)
         self.margin = margin
@@ -658,7 +673,7 @@ class _TickBar(qt.QWidget):
 
         self._vmin = vmin
         self._vmax = vmax
-        self._norm = norm
+        self._normalizer = normalizer
         self.displayValues = displayValues
         self.setTicksNumber(nticks)
 
@@ -675,10 +690,10 @@ class _TickBar(qt.QWidget):
         width = self._WIDTH_DISP_VAL if self.displayValues else self._WIDTH_NO_DISP_VAL
         self.setFixedWidth(width)
 
-    def update(self, vmin, vmax, norm):
+    def update(self, vmin, vmax, normalizer):
         self._vmin = vmin
         self._vmax = vmax
-        self._norm = norm
+        self._normalizer = normalizer
         self.computeTicks()
         qt.QWidget.update(self)
 
@@ -722,13 +737,11 @@ class _TickBar(qt.QWidget):
             # No range: no ticks
             self.ticks = ()
             self.subTicks = ()
-        elif self._norm == Colormap.Colormap.LOGARITHM:
+        elif isinstance(self._normalizer, colors._LogarithmicNormalization):
             self._computeTicksLog(nticks)
-        elif self._norm == Colormap.Colormap.LINEAR:
+        else:  # Fallback: use linear
             self._computeTicksLin(nticks)
-        else:
-            err = 'TickBar - Wrong normalization %s' % self._norm
-            raise ValueError(err)
+
         # update the form
         font = qt.QFont()
         font.setPixelSize(_TickBar._FONT_SIZE)
@@ -781,12 +794,17 @@ class _TickBar(qt.QWidget):
     def _getRelativePosition(self, val):
         """Return the relative position of val according to min and max value
         """
-        if self._norm == Colormap.Colormap.LINEAR:
-            return 1 - (val - self._vmin) / (self._vmax - self._vmin)
-        elif self._norm == Colormap.Colormap.LOGARITHM:
-            return 1 - (numpy.log10(val) - numpy.log10(self._vmin))/(numpy.log10(self._vmax) - numpy.log(self._vmin))
+        if self._normalizer is None:
+            return 0.
+        normMin, normMax, normVal = self._normalizer.apply(
+            [self._vmin, self._vmax, val],
+            self._vmin,
+            self._vmax)
+
+        if normMin == normMax:
+            return 0.
         else:
-            raise ValueError('Norm is not recognized')
+            return 1. - (normVal - normMin) / (normMax - normMin)
 
     def _paintTick(self, val, painter, majorTick=True):
         """
@@ -797,19 +815,18 @@ class _TickBar(qt.QWidget):
         fm = qt.QFontMetrics(painter.font())
         viewportHeight = self.rect().height() - self.margin * 2 - 1
         relativePos = self._getRelativePosition(val)
-        height = viewportHeight * relativePos
-        height += self.margin
+        height = int(viewportHeight * relativePos + self.margin)
         lineWidth = _TickBar._LINE_WIDTH
         if majorTick is False:
             lineWidth /= 2
 
-        painter.drawLine(qt.QLine(self.width() - lineWidth,
+        painter.drawLine(qt.QLine(int(self.width() - lineWidth),
                                   height,
                                   self.width(),
                                   height))
 
         if self.displayValues and majorTick is True:
-            painter.drawText(qt.QPoint(0.0, height + (fm.height() / 2)),
+            painter.drawText(qt.QPoint(0, int(height + fm.height() / 2)),
                              self.form.format(val))
 
     def setDisplayType(self, disType):
@@ -833,9 +850,9 @@ class _TickBar(qt.QWidget):
     def _getFormat(self, font):
         if self._forcedDisplayType is None:
             return self._guessType(font)
-        elif self._forcedDisplayType is 'std':
+        elif self._forcedDisplayType == 'std':
             return self._getStandardFormat()
-        elif self._forcedDisplayType is 'e':
+        elif self._forcedDisplayType == 'e':
             return self._getScientificForm()
         else:
             err = 'Forced type for display %s is not recognized' % self._forcedDisplayType
@@ -847,16 +864,16 @@ class _TickBar(qt.QWidget):
     def _guessType(self, font):
         """Try fo find the better format to display the tick's labels
 
-        :param QFont font: the font we want want to use durint the painting
+        :param QFont font: the font we want to use during the painting
         """
         form = self._getStandardFormat()
 
         fm = qt.QFontMetrics(font)
         width = 0
         for tick in self.ticks:
-            width = max(fm.width(form.format(tick)), width)
+            width = max(fm.boundingRect(form.format(tick)).width(), width)
 
-        # if the length of the string are too long we are mooving to scientific
+        # if the length of the string are too long we are moving to scientific
         # display
         if width > _TickBar._WIDTH_DISP_VAL - _TickBar._LINE_WIDTH:
             return self._getScientificForm()
