@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2017-2018 European Synchrotron Radiation Facility
+# Copyright (c) 2017-2020 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -32,36 +32,54 @@ This widget is meant to work with :class:`silx.gui.plot.PlotWidget`.
 """
 from __future__ import division
 
-
 __authors__ = ["T. Vincent", "P. Knobel"]
 __license__ = "MIT"
-__date__ = "24/04/2018"
-
+__date__ = "08/12/2020"
 
 import os
 import sys
 import numpy
 import logging
 import collections
+import h5py
 
 from silx.image import shapes
+from silx.io.utils import NEXUS_HDF5_EXT, is_dataset
+from silx.gui.dialog.DatasetDialog import DatasetDialog
 
 from ._BaseMaskToolsWidget import BaseMask, BaseMaskToolsWidget, BaseMaskToolsDockWidget
 from . import items
 from ..colors import cursorColorForColormap, rgba
 from .. import qt
+from ..utils import LockReentrant
 
 from silx.third_party.EdfFile import EdfFile
 from silx.third_party.TiffIO import TiffIO
 
-try:
-    import fabio
-except ImportError:
-    fabio = None
-
+import fabio
 
 _logger = logging.getLogger(__name__)
 
+_HDF5_EXT_STR = ' '.join(['*' + ext for ext in NEXUS_HDF5_EXT])
+
+
+def _selectDataset(filename, mode=DatasetDialog.SaveMode):
+    """Open a dialog to prompt the user to select a dataset in
+    a hdf5 file.
+
+    :param str filename: name of an existing HDF5 file
+    :param mode: DatasetDialog.SaveMode or DatasetDialog.LoadMode
+    :rtype: str
+    :return: Name of selected dataset
+    """
+    dialog = DatasetDialog()
+    dialog.addFile(filename)
+    dialog.setWindowTitle("Select a 2D dataset")
+    dialog.setMode(mode)
+    if not dialog.exec_():
+        return None
+    return dialog.getSelectedDataUrl().data_path()
+
 
 class ImageMask(BaseMask):
     """A 2D mask field with update operations.
@@ -70,6 +88,7 @@ class ImageMask(BaseMask):
 
     This is meant for internal use by :class:`MaskToolsWidget`.
     """
+
     def __init__(self, image=None):
         """
 
@@ -89,13 +108,14 @@ class ImageMask(BaseMask):
         """Save current mask in a file
 
         :param str filename: The file where to save to mask
-        :param str kind: The kind of file to save in 'edf', 'tif', 'npy',
+        :param str kind: The kind of file to save in 'edf', 'tif', 'npy', 'h5'
             or 'msk' (if FabIO is installed)
         :raise Exception: Raised if the file writing fail
         """
         if kind == 'edf':
             edfFile = EdfFile(filename, access="w+")
-            edfFile.WriteImage({}, self.getMask(copy=False), Append=0)
+            header = {"program_name": "silx-mask", "masked_value": "nonzero"}
+            edfFile.WriteImage(header, self.getMask(copy=False), Append=0)
 
         elif kind == 'tif':
             tiffFile = TiffIO(filename, mode='w')
@@ -107,9 +127,10 @@ class ImageMask(BaseMask):
             except IOError:
                 raise RuntimeError("Mask file can't be written")
 
+        elif ("." + kind) in NEXUS_HDF5_EXT:
+            self._saveToHdf5(filename, self.getMask(copy=False))
+
         elif kind == 'msk':
-            if fabio is None:
-                raise ImportError("Fit2d mask files can't be written: Fabio module is not available")
             try:
                 data = self.getMask(copy=False)
                 image = fabio.fabioimage.FabioImage(data=data)
@@ -118,10 +139,41 @@ class ImageMask(BaseMask):
             except Exception:
                 _logger.debug("Backtrace", exc_info=True)
                 raise RuntimeError("Mask file can't be written")
-
         else:
             raise ValueError("Format '%s' is not supported" % kind)
 
+    @staticmethod
+    def _saveToHdf5(filename, mask):
+        """Save a mask array to a HDF5 file.
+
+        :param str filename: name of an existing HDF5 file
+        :param numpy.ndarray mask: Mask array.
+        :returns: True if operation succeeded, False otherwise.
+        """
+        if not os.path.exists(filename):
+            # create new file
+            with h5py.File(filename, "w") as _h5f:
+                pass
+        dataPath = _selectDataset(filename)
+        if dataPath is None:
+            return False
+        with h5py.File(filename, "a") as h5f:
+            existing_ds = h5f.get(dataPath)
+            if existing_ds is not None:
+                reply = qt.QMessageBox.question(
+                        None,
+                        "Confirm overwrite",
+                        "Do you want to overwrite an existing dataset?",
+                        qt.QMessageBox.Yes | qt.QMessageBox.No)
+                if reply != qt.QMessageBox.Yes:
+                    return False
+                del h5f[dataPath]
+            try:
+                h5f.create_dataset(dataPath, data=mask)
+            except Exception:
+                return False
+        return True
+
     # Drawing operations
     def updateRectangle(self, level, row, col, height, width, mask=True):
         """Mask/Unmask a rectangle of the given mask level.
@@ -134,10 +186,12 @@ class ImageMask(BaseMask):
         :param bool mask: True to mask (default), False to unmask.
         """
         assert 0 < level < 256
+        if row + height <= 0 or col + width <= 0:
+            return  # Rectangle outside image, avoid negative indices
         selection = self._mask[max(0, row):row + height + 1,
                                max(0, col):col + width + 1]
         if mask:
-            selection[:, :] = level
+            selection[:,:] = level
         else:
             selection[selection == level] = 0
         self._notify()
@@ -192,6 +246,19 @@ class ImageMask(BaseMask):
         rows, cols = shapes.circle_fill(crow, ccol, radius)
         self.updatePoints(level, rows, cols, mask)
 
+    def updateEllipse(self, level, crow, ccol, radius_r, radius_c, mask=True):
+        """Mask/Unmask an ellipse of the given mask level.
+
+        :param int level: Mask level to update.
+        :param int crow: Row of the center of the ellipse
+        :param int ccol: Column of the center of the ellipse
+        :param float radius_r: Radius of the ellipse in the row
+        :param float radius_c: Radius of the ellipse in the column
+        :param bool mask: True to mask (default), False to unmask.
+        """
+        rows, cols = shapes.ellipse_fill(crow, ccol, radius_r, radius_c)
+        self.updatePoints(level, rows, cols, mask)
+
     def updateLine(self, level, row0, col0, row1, col1, width, mask=True):
         """Mask/Unmask a line of the given mask level.
 
@@ -220,6 +287,38 @@ class MaskToolsWidget(BaseMaskToolsWidget):
         self._z = 1  # Mask layer in plot
         self._data = numpy.zeros((0, 0), dtype=numpy.uint8)  # Store image
 
+        self.__itemMaskUpdatedLock = LockReentrant()
+        self.__itemMaskUpdated = False
+
+    def __maskStateChanged(self) -> None:
+        """Handle mask commit to update item mask"""
+        item = self._mask.getDataItem()
+        if item is not None:
+            with self.__itemMaskUpdatedLock:
+                item.setMaskData(self._mask.getMask(copy=True), copy=False)
+
+    def setItemMaskUpdated(self, enabled: bool) -> None:
+        """Toggle item mask and mask tool synchronisation.
+
+        :param bool enabled: True to synchronise. Default: False
+        """
+        enabled = bool(enabled)
+        if enabled != self.__itemMaskUpdated:
+            if self.__itemMaskUpdated:
+                self._mask.sigStateChanged.disconnect(self.__maskStateChanged)
+            self.__itemMaskUpdated = enabled
+            if self.__itemMaskUpdated:
+                # Synchronize item and tool mask
+                self._setMaskedImage(self._mask.getDataItem())
+                self._mask.sigStateChanged.connect(self.__maskStateChanged)
+
+    def isItemMaskUpdated(self) -> bool:
+        """Returns whether or not item and mask tool masks are synchronised.
+
+        :rtype: bool
+        """
+        return self.__itemMaskUpdated
+
     def setSelectionMask(self, mask, copy=True):
         """Set the mask to a new array.
 
@@ -242,17 +341,14 @@ class MaskToolsWidget(BaseMaskToolsWidget):
             _logger.error('Not an image, shape: %d', len(mask.shape))
             return None
 
+        # Handle mask with single level
+        if self.multipleMasks() == 'single':
+            mask = numpy.array(mask != 0, dtype=numpy.uint8)
+
         # if mask has not changed, do nothing
         if numpy.array_equal(mask, self.getSelectionMask()):
             return mask.shape
 
-        # ensure all mask attributes are synchronized with the active image
-        # and connect listener
-        activeImage = self.plot.getActiveImage()
-        if activeImage is not None and activeImage.getLegend() != self._maskName:
-            self._activeImageChanged()
-            self.plot.sigActiveImageChanged.connect(self._activeImageChanged)
-
         if self._data.shape[0:2] == (0, 0) or mask.shape == self._data.shape[0:2]:
             self._mask.setMask(mask, copy=copy)
             self._mask.commit()
@@ -266,7 +362,7 @@ class MaskToolsWidget(BaseMaskToolsWidget):
                                       dtype=numpy.uint8)
             height = min(self._data.shape[0], mask.shape[0])
             width = min(self._data.shape[1], mask.shape[1])
-            resizedMask[:height, :width] = mask[:height, :width]
+            resizedMask[:height,:width] = mask[:height,:width]
             self._mask.setMask(resizedMask, copy=False)
             self._mask.commit()
             return resizedMask.shape
@@ -281,7 +377,7 @@ class MaskToolsWidget(BaseMaskToolsWidget):
             mustBeAdded = maskItem is None
             if mustBeAdded:
                 maskItem = items.MaskImageData()
-                maskItem._setLegend(self._maskName)
+                maskItem.setName(self._maskName)
             # update the items
             maskItem.setData(mask, copy=False)
             maskItem.setColormap(self._colormap)
@@ -290,7 +386,7 @@ class MaskToolsWidget(BaseMaskToolsWidget):
             maskItem.setZValue(self._z)
 
             if mustBeAdded:
-                self.plot._add(maskItem)
+                self.plot.addItem(maskItem)
 
         elif self.plot.getImage(self._maskName):
             self.plot.remove(self._maskName, kind='image')
@@ -301,7 +397,9 @@ class MaskToolsWidget(BaseMaskToolsWidget):
                 self._activeImageChangedAfterCare)
         except (RuntimeError, TypeError):
             pass
-        self._activeImageChanged()  # Init mask + enable/disable widget
+
+        # Sync with current active image
+        self._setMaskedImage(self.plot.getActiveImage())
         self.plot.sigActiveImageChanged.connect(self._activeImageChanged)
 
     def hideEvent(self, event):
@@ -310,13 +408,41 @@ class MaskToolsWidget(BaseMaskToolsWidget):
                 self._activeImageChanged)
         except (RuntimeError, TypeError):
             pass
-        if not self.browseAction.isChecked():
-            self.browseAction.trigger()  # Disable drawing tool
 
-        if self.getSelectionMask(copy=False) is not None:
+        image = self.getMaskedItem()
+        if image is not None:
+            try:
+                image.sigItemChanged.disconnect(self.__imageChanged)
+            except (RuntimeError, TypeError):
+                pass  # TODO should not happen
+
+        if self.isMaskInteractionActivated():
+            # Disable drawing tool
+            self.browseAction.trigger()
+
+        if self.isItemMaskUpdated():  # No "after-care"
+            self._data = numpy.zeros((0, 0), dtype=numpy.uint8)
+            self._mask.setDataItem(None)
+            self._mask.reset()
+
+            if self.plot.getImage(self._maskName):
+                self.plot.remove(self._maskName, kind='image')
+
+        elif self.getSelectionMask(copy=False) is not None:
             self.plot.sigActiveImageChanged.connect(
                 self._activeImageChangedAfterCare)
 
+    def _activeImageChanged(self, previous, current):
+        """Reacts upon active image change.
+
+        Only handle change of active image items here.
+        """
+        if previous != current:
+            image = self.plot.getActiveImage()
+            if image is not None and image.getName() == self._maskName:
+                image = None  # Active image is the mask
+            self._setMaskedImage(image)
+
     def _setOverlayColorForImage(self, image):
         """Set the color of overlay adapted to image
 
@@ -336,7 +462,7 @@ class MaskToolsWidget(BaseMaskToolsWidget):
         removed, otherwise it is adjusted to origin, scale and z.
         """
         activeImage = self.plot.getActiveImage()
-        if activeImage is None or activeImage.getLegend() == self._maskName:
+        if activeImage is None or activeImage.getName() == self._maskName:
             # No active image or active image is the mask...
             self._data = numpy.zeros((0, 0), dtype=numpy.uint8)
             self._mask.setDataItem(None)
@@ -369,41 +495,93 @@ class MaskToolsWidget(BaseMaskToolsWidget):
                 self._mask.setDataItem(activeImage)
                 self._updatePlotMask()
 
-    def _activeImageChanged(self, *args):
-        """Update widget and mask according to active image changes"""
-        activeImage = self.plot.getActiveImage()
-        if (activeImage is None or activeImage.getLegend() == self._maskName or
-                activeImage.getData(copy=False).size == 0):
-            # No active image or active image is the mask or image has no data...
+    def _setMaskedImage(self, image):
+        """Change the image that is used a reference to author the mask"""
+        previous = self.getMaskedItem()
+        if previous is not None and self.isVisible():
+            # Disconnect from previous image
+            try:
+                previous.sigItemChanged.disconnect(self.__imageChanged)
+            except TypeError:
+                pass  # TODO fixme should not happen
+
+        # Set the image
+        self._mask.setDataItem(image)
+
+        if image is None:  # No image, disable mask
             self.setEnabled(False)
 
             self._data = numpy.zeros((0, 0), dtype=numpy.uint8)
             self._mask.reset()
             self._mask.commit()
 
-        else:  # There is an active image
-            self.setEnabled(True)
+            self._updateInteractiveMode()
 
-            self._setOverlayColorForImage(activeImage)
+        else:  # Update and connect to image's sigItemChanged
+            if self.isItemMaskUpdated():
+                if image.getMaskData(copy=False) is None:
+                    # Image item has no mask: use current mask from the tool
+                    image.setMaskData(
+                        self.getSelectionMask(copy=False), copy=True)
+                else:  # Image item has a mask: set it in tool
+                    self.setSelectionMask(
+                        image.getMaskData(copy=False), copy=True)
+                    self._mask.resetHistory()
+            self.__imageUpdated()
+            if self.isVisible():
+                image.sigItemChanged.connect(self.__imageChanged)
 
-            self._setMaskColors(self.levelSpinBox.value(),
-                                self.transparencySlider.value() /
-                                self.transparencySlider.maximum())
+    def __imageChanged(self, event):
+        """Reacts upon image item changes"""
+        image = self._mask.getDataItem()
+        if image is None:
+            _logger.error("Mask is not attached to an image")
+            return
 
-            self._origin = activeImage.getOrigin()
-            self._scale = activeImage.getScale()
-            self._z = activeImage.getZValue() + 1
-            self._data = activeImage.getData(copy=False)
-            self._mask.setDataItem(activeImage)
-            if self._data.shape[:2] != self._mask.getMask(copy=False).shape:
-                self._mask.reset(self._data.shape[:2])
-                self._mask.commit()
-            else:
-                # Refresh in case origin, scale, z changed
-                self._updatePlotMask()
+        if event in (items.ItemChangedType.COLORMAP,
+                     items.ItemChangedType.DATA,
+                     items.ItemChangedType.POSITION,
+                     items.ItemChangedType.SCALE,
+                     items.ItemChangedType.VISIBLE,
+                     items.ItemChangedType.ZVALUE):
+            self.__imageUpdated()
 
-            # Threshold tools only available for data with colormap
-            self.thresholdGroup.setEnabled(self._data.ndim == 2)
+        elif (event == items.ItemChangedType.MASK and
+                self.isItemMaskUpdated() and
+                not self.__itemMaskUpdatedLock.locked()):
+            # Update mask from the image item unless mask tool is updating it
+            self.setSelectionMask(image.getMaskData(copy=False), copy=True)
+
+    def __imageUpdated(self):
+        """Synchronize mask with current state of the image"""
+        image = self._mask.getDataItem()
+        if image is None:
+            _logger.error("No active image while expecting one")
+            return
+
+        self._setOverlayColorForImage(image)
+
+        self._setMaskColors(self.levelSpinBox.value(),
+                            self.transparencySlider.value() /
+                            self.transparencySlider.maximum())
+
+        self._origin = image.getOrigin()
+        self._scale = image.getScale()
+        self._z = image.getZValue() + 1
+        self._data = image.getData(copy=False)
+        self._mask.setDataItem(image)
+        if self._data.shape[:2] != self._mask.getMask(copy=False).shape:
+            self._mask.reset(self._data.shape[:2])
+            self._mask.commit()
+        else:
+            # Refresh in case origin, scale, z changed
+            self._updatePlotMask()
+
+        # Visible and with data
+        self.setEnabled(image.isVisible() and self._data.size != 0)
+
+        # Threshold tools only available for data with colormap
+        self.thresholdGroup.setEnabled(self._data.ndim == 2)
 
         self._updateInteractiveMode()
 
@@ -442,14 +620,16 @@ class MaskToolsWidget(BaseMaskToolsWidget):
                 _logger.debug("Backtrace", exc_info=True)
                 raise e
         elif extension == "msk":
-            if fabio is None:
-                raise ImportError("Fit2d mask files can't be read: Fabio module is not available")
             try:
                 mask = fabio.open(filename).data
             except Exception as e:
                 _logger.error("Can't load fit2d mask file")
                 _logger.debug("Backtrace", exc_info=True)
                 raise e
+        elif ("." + extension) in NEXUS_HDF5_EXT:
+            mask = self._loadFromHdf5(filename)
+            if mask is None:
+                raise IOError("Could not load mask from HDF5 dataset")
         else:
             msg = "Extension '%s' is not supported."
             raise RuntimeError(msg % extension)
@@ -472,6 +652,7 @@ class MaskToolsWidget(BaseMaskToolsWidget):
         extensions["EDF files"] = "*.edf"
         extensions["TIFF files"] = "*.tif *.tiff"
         extensions["NumPy binary files"] = "*.npy"
+        extensions["HDF5 files"] = _HDF5_EXT_STR
         # Fit2D mask is displayed anyway fabio is here or not
         # to show to the user that the option exists
         extensions["Fit2D mask files"] = "*.msk"
@@ -492,7 +673,9 @@ class MaskToolsWidget(BaseMaskToolsWidget):
         filename = dialog.selectedFiles()[0]
         dialog.close()
 
+        # Update the directory according to the user selection
         self.maskFileDir = os.path.dirname(filename)
+
         try:
             self.load(filename)
         except RuntimeWarning as e:
@@ -508,15 +691,37 @@ class MaskToolsWidget(BaseMaskToolsWidget):
             msg.setText("Cannot load mask from file. " + message)
             msg.exec_()
 
+    @staticmethod
+    def _loadFromHdf5(filename):
+        """Load a mask array from a HDF5 file.
+
+        :param str filename: name of an existing HDF5 file
+        :returns: A mask as a numpy array, or None if the interactive dialog
+            was cancelled
+        """
+        dataPath = _selectDataset(filename, mode=DatasetDialog.LoadMode)
+        if dataPath is None:
+            return None
+
+        with h5py.File(filename, "r") as h5f:
+            dataset = h5f.get(dataPath)
+            if not is_dataset(dataset):
+                raise IOError("%s is not a dataset" % dataPath)
+            mask = dataset[()]
+        return mask
+
     def _saveMask(self):
         """Open Save mask dialog"""
         dialog = qt.QFileDialog(self)
         dialog.setWindowTitle("Save Mask")
+        dialog.setOption(dialog.DontUseNativeDialog)
         dialog.setModal(1)
+        hdf5Filter = 'HDF5 (%s)' % _HDF5_EXT_STR
         filters = [
             'EDF (*.edf)',
             'TIFF (*.tif)',
             'NumPy binary file (*.npy)',
+            hdf5Filter,
             # Fit2D mask is displayed anyway fabio is here or not
             # to show to the user that the option exists
             'Fit2D mask (*.msk)',
@@ -525,36 +730,72 @@ class MaskToolsWidget(BaseMaskToolsWidget):
         dialog.setFileMode(qt.QFileDialog.AnyFile)
         dialog.setAcceptMode(qt.QFileDialog.AcceptSave)
         dialog.setDirectory(self.maskFileDir)
+
+        def onFilterSelection(filt_):
+            # disable overwrite confirmation for HDF5,
+            # because we append the data to existing files
+            if filt_ == hdf5Filter:
+                dialog.setOption(dialog.DontConfirmOverwrite)
+            else:
+                dialog.setOption(dialog.DontConfirmOverwrite, False)
+
+        dialog.filterSelected.connect(onFilterSelection)
         if not dialog.exec_():
             dialog.close()
             return
 
-        # convert filter name to extension name with the .
-        extension = dialog.selectedNameFilter().split()[-1][2:-1]
+        nameFilter = dialog.selectedNameFilter()
         filename = dialog.selectedFiles()[0]
         dialog.close()
 
-        if not filename.lower().endswith(extension):
-            filename += extension
+        if "HDF5" in nameFilter:
+            has_allowed_ext = False
+            for ext in NEXUS_HDF5_EXT:
+                if (len(filename) > len(ext) and
+                        filename[-len(ext):].lower() == ext.lower()):
+                    has_allowed_ext = True
+                    extension = ext
+            if not has_allowed_ext:
+                extension = ".h5"
+                filename += ".h5"
+        else:
+            # convert filter name to extension name with the .
+            extension = nameFilter.split()[-1][2:-1]
+            if not filename.lower().endswith(extension):
+                filename += extension
 
-        if os.path.exists(filename):
+        if os.path.exists(filename) and "HDF5" not in nameFilter:
             try:
                 os.remove(filename)
-            except IOError:
+            except IOError as e:
                 msg = qt.QMessageBox(self)
+                msg.setWindowTitle("Removing existing file")
                 msg.setIcon(qt.QMessageBox.Critical)
+
+                if hasattr(e, "strerror"):
+                    strerror = e.strerror
+                else:
+                    strerror = sys.exc_info()[1]
                 msg.setText("Cannot save.\n"
-                            "Input Output Error: %s" % (sys.exc_info()[1]))
+                            "Input Output Error: %s" % strerror)
                 msg.exec_()
                 return
 
+        # Update the directory according to the user selection
         self.maskFileDir = os.path.dirname(filename)
+
         try:
             self.save(filename, extension[1:])
         except Exception as e:
             msg = qt.QMessageBox(self)
+            msg.setWindowTitle("Saving mask file")
             msg.setIcon(qt.QMessageBox.Critical)
-            msg.setText("Cannot save file %s\n%s" % (filename, e.args[0]))
+
+            if hasattr(e, "strerror"):
+                strerror = e.strerror
+            else:
+                strerror = sys.exc_info()[1]
+            msg.setText("Cannot save file %s\n%s" % (filename, strerror))
             msg.exec_()
 
     def resetSelectionMask(self):
@@ -573,41 +814,51 @@ class MaskToolsWidget(BaseMaskToolsWidget):
 
         level = self.levelSpinBox.value()
 
-        if (self._drawingMode == 'rectangle' and
-                event['event'] == 'drawingFinished'):
-            # Convert from plot to array coords
-            doMask = self._isMasking()
-            ox, oy = self._origin
-            sx, sy = self._scale
+        if self._drawingMode == 'rectangle':
+            if event['event'] == 'drawingFinished':
+                # Convert from plot to array coords
+                doMask = self._isMasking()
+                ox, oy = self._origin
+                sx, sy = self._scale
 
-            height = int(abs(event['height'] / sy))
-            width = int(abs(event['width'] / sx))
+                height = int(abs(event['height'] / sy))
+                width = int(abs(event['width'] / sx))
 
-            row = int((event['y'] - oy) / sy)
-            if sy < 0:
-                row -= height
+                row = int((event['y'] - oy) / sy)
+                if sy < 0:
+                    row -= height
 
-            col = int((event['x'] - ox) / sx)
-            if sx < 0:
-                col -= width
+                col = int((event['x'] - ox) / sx)
+                if sx < 0:
+                    col -= width
 
-            self._mask.updateRectangle(
-                level,
-                row=row,
-                col=col,
-                height=height,
-                width=width,
-                mask=doMask)
-            self._mask.commit()
+                self._mask.updateRectangle(
+                    level,
+                    row=row,
+                    col=col,
+                    height=height,
+                    width=width,
+                    mask=doMask)
+                self._mask.commit()
 
-        elif (self._drawingMode == 'polygon' and
-                event['event'] == 'drawingFinished'):
-            doMask = self._isMasking()
-            # Convert from plot to array coords
-            vertices = (event['points'] - self._origin) / self._scale
-            vertices = vertices.astype(numpy.int)[:, (1, 0)]  # (row, col)
-            self._mask.updatePolygon(level, vertices, doMask)
-            self._mask.commit()
+        elif self._drawingMode == 'ellipse':
+            if event['event'] == 'drawingFinished':
+                doMask = self._isMasking()
+                # Convert from plot to array coords
+                center = (event['points'][0] - self._origin) / self._scale
+                size = event['points'][1] / self._scale
+                center = center.astype(numpy.int64)  # (row, col)
+                self._mask.updateEllipse(level, center[1], center[0], size[1], size[0], doMask)
+                self._mask.commit()
+
+        elif self._drawingMode == 'polygon':
+            if event['event'] == 'drawingFinished':
+                doMask = self._isMasking()
+                # Convert from plot to array coords
+                vertices = (event['points'] - self._origin) / self._scale
+                vertices = vertices.astype(numpy.int64)[:, (1, 0)]  # (row, col)
+                self._mask.updatePolygon(level, vertices, doMask)
+                self._mask.commit()
 
         elif self._drawingMode == 'pencil':
             doMask = self._isMasking()
@@ -634,12 +885,14 @@ class MaskToolsWidget(BaseMaskToolsWidget):
                 self._lastPencilPos = None
             else:
                 self._lastPencilPos = row, col
+        else:
+            _logger.error("Drawing mode %s unsupported", self._drawingMode)
 
     def _loadRangeFromColormapTriggered(self):
         """Set range from active image colormap range"""
         activeImage = self.plot.getActiveImage()
         if (isinstance(activeImage, items.ColormapMixIn) and
-                activeImage.getLegend() != self._maskName):
+                activeImage.getName() != self._maskName):
             # Update thresholds according to colormap
             colormap = activeImage.getColormap()
             if colormap['autoscale']:
@@ -660,6 +913,7 @@ class MaskToolsDockWidget(BaseMaskToolsDockWidget):
     :param plot: The PlotWidget this widget is operating on
     :paran str name: The title of this widget
     """
+
     def __init__(self, parent=None, plot=None, name='Mask'):
         widget = MaskToolsWidget(plot=plot)
         super(MaskToolsDockWidget, self).__init__(parent, name, widget)
