@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2017 European Synchrotron Radiation Facility
+# Copyright (c) 2017-2021 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -28,18 +28,19 @@ of the :class:`Plot`.
 
 __authors__ = ["T. Vincent"]
 __license__ = "MIT"
-__date__ = "27/06/2017"
+__date__ = "08/12/2020"
 
-
-from collections import Sequence
+try:
+    from collections import abc
+except ImportError:  # Python2 support
+    import collections as abc
 import logging
 
 import numpy
 
-from .core import (Item, LabelsMixIn, DraggableMixIn, ColormapMixIn,
+from ....utils.proxy import docstring
+from .core import (DataItem, LabelsMixIn, DraggableMixIn, ColormapMixIn,
                    AlphaMixIn, ItemChangedType)
-from ..Colors import applyColormapToData
-
 
 _logger = logging.getLogger(__name__)
 
@@ -62,7 +63,7 @@ def _convertImageToRgba32(image, copy=True):
     assert image.shape[-1] in (3, 4)
 
     # Convert type to uint8
-    if image.dtype.name != 'uin8':
+    if image.dtype.name != 'uint8':
         if image.dtype.kind == 'f':  # Float in [0, 1]
             image = (numpy.clip(image, 0., 1.) * 255).astype(numpy.uint8)
         elif image.dtype.kind == 'b':  # boolean
@@ -77,23 +78,29 @@ def _convertImageToRgba32(image, copy=True):
     if image.shape[-1] == 3:
         new_image = numpy.empty((image.shape[0], image.shape[1], 4),
                                 dtype=numpy.uint8)
-        new_image[:, :, :3] = image
-        new_image[:, :, 3] = 255
+        new_image[:,:,:3] = image
+        new_image[:,:, 3] = 255
         return new_image  # This is a copy anyway
     else:
         return numpy.array(image, copy=copy)
 
 
-class ImageBase(Item, LabelsMixIn, DraggableMixIn, AlphaMixIn):
-    """Description of an image"""
+class ImageBase(DataItem, LabelsMixIn, DraggableMixIn, AlphaMixIn):
+    """Description of an image
 
-    def __init__(self):
-        Item.__init__(self)
+    :param numpy.ndarray data: Initial image data
+    """
+
+    def __init__(self, data=None, mask=None):
+        DataItem.__init__(self)
         LabelsMixIn.__init__(self)
         DraggableMixIn.__init__(self)
         AlphaMixIn.__init__(self)
-        self._data = numpy.zeros((0, 0, 4), dtype=numpy.uint8)
-
+        if data is None:
+            data = numpy.zeros((0, 0, 4), dtype=numpy.uint8)
+        self._data = data
+        self._mask = mask
+        self.__valueDataCache = None  # Store default data
         self._origin = (0., 0.)
         self._scale = (1., 1.)
 
@@ -104,7 +111,7 @@ class ImageBase(Item, LabelsMixIn, DraggableMixIn, AlphaMixIn):
         elif item == 0:
             return self.getData(copy=False)
         elif item == 1:
-            return self.getLegend()
+            return self.getName()
         elif item == 2:
             info = self.getInfo(copy=False)
             return {} if info is None else info
@@ -126,19 +133,6 @@ class ImageBase(Item, LabelsMixIn, DraggableMixIn, AlphaMixIn):
         else:
             raise IndexError("Index out of range: %s" % str(item))
 
-    def setVisible(self, visible):
-        """Set visibility of item.
-
-        :param bool visible: True to display it, False otherwise
-        """
-        visible = bool(visible)
-        # TODO hackish data range implementation
-        if self.isVisible() != visible:
-            plot = self.getPlot()
-            if plot is not None:
-                plot._invalidateDataRange()
-        super(ImageBase, self).setVisible(visible)
-
     def _isPlotLinear(self, plot):
         """Return True if plot only uses linear scale for both of x and y
         axes."""
@@ -171,6 +165,12 @@ class ImageBase(Item, LabelsMixIn, DraggableMixIn, AlphaMixIn):
         else:
             return xmin, xmax, ymin, ymax
 
+    @docstring(DraggableMixIn)
+    def drag(self, from_, to):
+        origin = self.getOrigin()
+        self.setOrigin((origin[0] + to[0] - from_[0],
+                        origin[1] + to[1] - from_[1]))
+
     def getData(self, copy=True):
         """Returns the image data
 
@@ -180,9 +180,103 @@ class ImageBase(Item, LabelsMixIn, DraggableMixIn, AlphaMixIn):
         """
         return numpy.array(self._data, copy=copy)
 
+    def setData(self, data):
+        """Set the image data
+
+        :param numpy.ndarray data:
+        """
+        previousShape = self._data.shape
+        self._data = data
+        self._valueDataChanged()
+        self._boundsChanged()
+        self._updated(ItemChangedType.DATA)
+
+        if (self.getMaskData(copy=False) is not None and
+                previousShape != self._data.shape):
+            # Data shape changed, so mask shape changes.
+            # Send event, mask is lazily updated in getMaskData
+            self._updated(ItemChangedType.MASK)
+
+    def getMaskData(self, copy=True):
+        """Returns the mask data
+
+        :param bool copy: True (Default) to get a copy,
+                          False to use internal representation (do not modify!)
+        :rtype: Union[None,numpy.ndarray]
+        """
+        if self._mask is None:
+            return None
+
+        # Update mask if it does not match data shape
+        shape = self.getData(copy=False).shape[:2]
+        if self._mask.shape != shape:
+            # Clip/extend mask to match data
+            newMask = numpy.zeros(shape, dtype=self._mask.dtype)
+            newMask[:self._mask.shape[0], :self._mask.shape[1]] = self._mask[:shape[0], :shape[1]]
+            self._mask = newMask
+
+        return numpy.array(self._mask, copy=copy)
+
+    def setMaskData(self, mask, copy=True):
+        """Set the image data
+
+        :param numpy.ndarray data:
+        :param bool copy: True (Default) to make a copy,
+                          False to use as is (do not modify!)
+        """
+        if mask is not None:
+            mask = numpy.array(mask, copy=copy)
+
+            shape = self.getData(copy=False).shape[:2]
+            if mask.shape != shape:
+                _logger.warning("Inconsistent shape between mask and data %s, %s", mask.shape, shape)
+                # Clip/extent is done lazily in getMaskData
+        elif self._mask is None:
+            return  # No update
+
+        self._mask = mask
+        self._valueDataChanged()
+        self._updated(ItemChangedType.MASK)
+
+    def _valueDataChanged(self):
+        """Clear cache of default data array"""
+        self.__valueDataCache = None
+
+    def _getValueData(self, copy=True):
+        """Return data used by :meth:`getValueData`
+
+        :param bool copy:
+        :rtype: numpy.ndarray
+        """
+        return self.getData(copy=copy)
+
+    def getValueData(self, copy=True):
+        """Return data (converted to int or float) with mask applied.
+
+        Masked values are set to Not-A-Number.
+        It returns a 2D array of values (int or float).
+
+        :param bool copy:
+        :rtype: numpy.ndarray
+        """
+        if self.__valueDataCache is None:
+            data = self._getValueData(copy=False)
+            mask = self.getMaskData(copy=False)
+            if mask is not None:
+                if numpy.issubdtype(data.dtype, numpy.floating):
+                    dtype = data.dtype
+                else:
+                    dtype = numpy.float64
+                data = numpy.array(data, dtype=dtype, copy=True)
+                data[mask != 0] = numpy.NaN
+            self.__valueDataCache = data
+        return numpy.array(self.__valueDataCache, copy=copy)
+
     def getRgbaImageData(self, copy=True):
         """Get the displayed RGB(A) image
 
+        :param bool copy: True (Default) to get a copy,
+                          False to use internal representation (do not modify!)
         :returns: numpy.ndarray of uint8 of shape (height, width, 4)
         """
         raise NotImplementedError('This MUST be implemented in sub-class')
@@ -200,19 +294,13 @@ class ImageBase(Item, LabelsMixIn, DraggableMixIn, AlphaMixIn):
         :param origin: (ox, oy) Offset from origin
         :type origin: float or 2-tuple of float
         """
-        if isinstance(origin, Sequence):
+        if isinstance(origin, abc.Sequence):
             origin = float(origin[0]), float(origin[1])
         else:  # single value origin
             origin = float(origin), float(origin)
         if origin != self._origin:
             self._origin = origin
-
-            # TODO hackish data range implementation
-            if self.isVisible():
-                plot = self.getPlot()
-                if plot is not None:
-                    plot._invalidateDataRange()
-
+            self._boundsChanged()
             self._updated(ItemChangedType.POSITION)
 
     def getScale(self):
@@ -228,20 +316,14 @@ class ImageBase(Item, LabelsMixIn, DraggableMixIn, AlphaMixIn):
         :param scale: (sx, sy) Scale of the image
         :type scale: float or 2-tuple of float
         """
-        if isinstance(scale, Sequence):
+        if isinstance(scale, abc.Sequence):
             scale = float(scale[0]), float(scale[1])
         else:  # single value scale
             scale = float(scale), float(scale)
 
         if scale != self._scale:
             self._scale = scale
-
-            # TODO hackish data range implementation
-            if self.isVisible():
-                plot = self.getPlot()
-                if plot is not None:
-                    plot._invalidateDataRange()
-
+            self._boundsChanged()
             self._updated(ItemChangedType.SCALE)
 
 
@@ -249,10 +331,10 @@ class ImageData(ImageBase, ColormapMixIn):
     """Description of a data image with a colormap"""
 
     def __init__(self):
-        ImageBase.__init__(self)
+        ImageBase.__init__(self, numpy.zeros((0, 0), dtype=numpy.float32))
         ColormapMixIn.__init__(self)
-        self._data = numpy.zeros((0, 0), dtype=numpy.float32)
         self._alternativeImage = None
+        self.__alpha = None
 
     def _addBackendRenderer(self, backend):
         """Update backend renderer"""
@@ -262,22 +344,25 @@ class ImageData(ImageBase, ColormapMixIn):
             # Do not render with non linear scales
             return None
 
-        if self.getAlternativeImageData(copy=False) is not None:
-            dataToUse = self.getAlternativeImageData(copy=False)
+        if (self.getAlternativeImageData(copy=False) is not None or
+                self.getAlphaData(copy=False) is not None):
+            dataToUse = self.getRgbaImageData(copy=False)
         else:
             dataToUse = self.getData(copy=False)
 
         if dataToUse.size == 0:
             return None  # No data to display
 
+        colormap = self.getColormap()
+        if colormap.isAutoscale():
+            # Avoid backend to compute autoscale: use item cache
+            colormap = colormap.copy()
+            colormap.setVRange(*colormap.getColormapRange(self))
+
         return backend.addImage(dataToUse,
-                                legend=self.getLegend(),
                                 origin=self.getOrigin(),
                                 scale=self.getScale(),
-                                z=self.getZValue(),
-                                selectable=self.isSelectable(),
-                                draggable=self.isDraggable(),
-                                colormap=self.getColormap(),
+                                colormap=colormap,
                                 alpha=self.getAlpha())
 
     def __getitem__(self, item):
@@ -294,37 +379,56 @@ class ImageData(ImageBase, ColormapMixIn):
     def getRgbaImageData(self, copy=True):
         """Get the displayed RGB(A) image
 
-        :returns: numpy.ndarray of uint8 of shape (height, width, 4)
+        :returns: Array of uint8 of shape (height, width, 4)
+        :rtype: numpy.ndarray
         """
-        if self._alternativeImage is not None:
-            return _convertImageToRgba32(
-                self.getAlternativeImageData(copy=False), copy=copy)
+        alternative = self.getAlternativeImageData(copy=False)
+        if alternative is not None:
+            return _convertImageToRgba32(alternative, copy=copy)
         else:
             # Apply colormap, in this case an new array is always returned
             colormap = self.getColormap()
-            image = colormap.applyToData(self.getData(copy=False))
+            image = colormap.applyToData(self)
+            alphaImage = self.getAlphaData(copy=False)
+            if alphaImage is not None:
+                # Apply transparency
+                image[:,:, 3] = image[:,:, 3] * alphaImage
             return image
 
     def getAlternativeImageData(self, copy=True):
         """Get the optional RGBA image that is displayed instead of the data
 
-        :param copy: True (Default) to get a copy,
-                     False to use internal representation (do not modify!)
-        :returns: None or numpy.ndarray
-        :rtype: numpy.ndarray or None
+        :param bool copy: True (Default) to get a copy,
+            False to use internal representation (do not modify!)
+        :rtype: Union[None,numpy.ndarray]
         """
         if self._alternativeImage is None:
             return None
         else:
             return numpy.array(self._alternativeImage, copy=copy)
 
-    def setData(self, data, alternative=None, copy=True):
+    def getAlphaData(self, copy=True):
+        """Get the optional transparency image applied on the data
+
+        :param bool copy: True (Default) to get a copy,
+            False to use internal representation (do not modify!)
+        :rtype: Union[None,numpy.ndarray]
+        """
+        if self.__alpha is None:
+            return None
+        else:
+            return numpy.array(self.__alpha, copy=copy)
+
+    def setData(self, data, alternative=None, alpha=None, copy=True):
         """"Set the image data and optionally an alternative RGB(A) representation
 
         :param numpy.ndarray data: Data array with 2 dimensions (h, w)
         :param alternative: RGB(A) image to display instead of data,
                             shape: (h, w, 3 or 4)
-        :type alternative: None or numpy.ndarray
+        :type alternative: Union[None,numpy.ndarray]
+        :param alpha: An array of transparency value in [0, 1] to use for
+                      display with shape: (h, w)
+        :type alpha: Union[None,numpy.ndarray]
         :param bool copy: True (Default) to get a copy,
                           False to use internal representation (do not modify!)
         """
@@ -334,11 +438,10 @@ class ImageData(ImageBase, ColormapMixIn):
             _logger.warning(
                 'Converting boolean image to int8 to plot it.')
             data = numpy.array(data, copy=False, dtype=numpy.int8)
-        elif numpy.issubdtype(data.dtype, numpy.complex):
+        elif numpy.iscomplexobj(data):
             _logger.warning(
                 'Converting complex image to absolute value to plot it.')
             data = numpy.absolute(data)
-        self._data = data
 
         if alternative is not None:
             alternative = numpy.array(alternative, copy=copy)
@@ -347,20 +450,31 @@ class ImageData(ImageBase, ColormapMixIn):
             assert alternative.shape[:2] == data.shape[:2]
         self._alternativeImage = alternative
 
-        # TODO hackish data range implementation
-        if self.isVisible():
-            plot = self.getPlot()
-            if plot is not None:
-                plot._invalidateDataRange()
+        if alpha is not None:
+            alpha = numpy.array(alpha, copy=copy)
+            assert alpha.shape == data.shape
+            if alpha.dtype.kind != 'f':
+                alpha = alpha.astype(numpy.float32)
+            if numpy.any(numpy.logical_or(alpha < 0., alpha > 1.)):
+                alpha = numpy.clip(alpha, 0., 1.)
+        self.__alpha = alpha
 
-        self._updated(ItemChangedType.DATA)
+        super().setData(data)
+
+    def _updated(self, event=None, checkVisibility=True):
+        # Synchronizes colormapped data if changed
+        if event in (ItemChangedType.DATA, ItemChangedType.MASK):
+            self._setColormappedData(
+                self.getValueData(copy=False),
+                copy=False)
+        super()._updated(event=event, checkVisibility=checkVisibility)
 
 
 class ImageRgba(ImageBase):
     """Description of an RGB(A) image"""
 
     def __init__(self):
-        ImageBase.__init__(self)
+        ImageBase.__init__(self, numpy.zeros((0, 0, 4), dtype=numpy.uint8))
 
     def _addBackendRenderer(self, backend):
         """Update backend renderer"""
@@ -376,12 +490,8 @@ class ImageRgba(ImageBase):
             return None  # No data to display
 
         return backend.addImage(data,
-                                legend=self.getLegend(),
                                 origin=self.getOrigin(),
                                 scale=self.getScale(),
-                                z=self.getZValue(),
-                                selectable=self.isSelectable(),
-                                draggable=self.isDraggable(),
                                 colormap=None,
                                 alpha=self.getAlpha())
 
@@ -402,15 +512,21 @@ class ImageRgba(ImageBase):
         data = numpy.array(data, copy=copy)
         assert data.ndim == 3
         assert data.shape[-1] in (3, 4)
-        self._data = data
+        super().setData(data)
 
-        # TODO hackish data range implementation
-        if self.isVisible():
-            plot = self.getPlot()
-            if plot is not None:
-                plot._invalidateDataRange()
+    def _getValueData(self, copy=True):
+        """Compute the intensity of the RGBA image as default data.
 
-        self._updated(ItemChangedType.DATA)
+        Conversion: https://en.wikipedia.org/wiki/YCbCr#ITU-R_BT.601_conversion
+
+        :param bool copy:
+        """
+        rgba = self.getRgbaImageData(copy=False).astype(numpy.float32)
+        intensity = (rgba[:, :, 0] * 0.299 +
+                     rgba[:, :, 1] * 0.587 +
+                     rgba[:, :, 2] * 0.114)
+        intensity *= rgba[:, :, 3] / 255.
+        return intensity
 
 
 class MaskImageData(ImageData):
@@ -420,3 +536,82 @@ class MaskImageData(ImageData):
     internal silx widgets.
     """
     pass
+
+
+class ImageStack(ImageData):
+    """Item to store a stack of images and to show it in the plot as one
+    of the images of the stack.
+
+    The stack is a 3D array ordered this way: `frame id, y, x`.
+    So the first image of the stack can be reached this way: `stack[0, :, :]`
+    """
+
+    def __init__(self):
+        ImageData.__init__(self)
+        self.__stack = None
+        """A 3D numpy array (or a mimic one, see ListOfImages)"""
+        self.__stackPosition = None
+        """Displayed position in the cube"""
+
+    def setStackData(self, stack, position=None, copy=True):
+        """Set the stack data
+
+        :param stack: A 3D numpy array like
+        :param int position: The position of the displayed image in the stack
+        :param bool copy: True (Default) to get a copy,
+                          False to use internal representation (do not modify!)
+        """
+        if self.__stack is stack:
+            return
+        if copy:
+            stack = numpy.array(stack)
+        assert stack.ndim == 3
+        self.__stack = stack
+        if position is not None:
+            self.__stackPosition = position
+        if self.__stackPosition is None:
+            self.__stackPosition = 0
+        self.__updateDisplayedData()
+
+    def getStackData(self, copy=True):
+        """Get the stored stack array.
+
+        :param bool copy: True (Default) to get a copy,
+                          False to use internal representation (do not modify!)
+        :rtype: A 3D numpy array, or numpy array like
+        """
+        if copy:
+            return numpy.array(self.__stack)
+        else:
+            return self.__stack
+
+    def setStackPosition(self, pos):
+        """Set the displayed position on the stack.
+
+        This function will clamp the stack position according to
+        the real size of the first axis of the stack.
+
+        :param int pos: A position on the first axis of the stack.
+        """
+        if self.__stackPosition == pos:
+            return
+        self.__stackPosition = pos
+        self.__updateDisplayedData()
+
+    def getStackPosition(self):
+        """Get the displayed position of the stack.
+
+        :rtype: int
+        """
+        return self.__stackPosition
+
+    def __updateDisplayedData(self):
+        """Update the displayed frame whenever the stack or the stack
+        position are updated."""
+        if self.__stack is None or self.__stackPosition is None:
+            empty = numpy.array([]).reshape(0, 0)
+            self.setData(empty, copy=False)
+            return
+        size = len(self.__stack)
+        self.__stackPosition = numpy.clip(self.__stackPosition, 0, size)
+        self.setData(self.__stack[self.__stackPosition], copy=False)
