@@ -24,7 +24,7 @@
 
 __authors__ = ["J. Kieffer"]
 __license__ = "MIT"
-__date__ = "02/08/2016"
+__date__ = "25/11/2020"
 
 import unittest
 import numpy
@@ -45,6 +45,10 @@ class TestBilinear(unittest.TestCase):
         gb = numpy.exp(-b * b / 6000)
         gg = numpy.outer(ga, gb)
         b = BilinearImage(gg)
+
+        self.assertAlmostEqual(b.maxi, 1, 2, "maxi is almost 1")
+        self.assertLess(b.mini, 0.3, "mini should be around 0.23")
+
         ok = 0
         for s in range(self.N):
             i, j = numpy.random.randint(100), numpy.random.randint(100)
@@ -78,24 +82,55 @@ class TestBilinear(unittest.TestCase):
         self.assertEqual(ok, self.N, "Maximum is always found")
 
     def test_map(self):
-        N = 100
-        y, x = numpy.ogrid[:N, :N + 10]
+        N = 6
+        y, x = numpy.ogrid[:N,:N + 10]
         img = x + y
         b = BilinearImage(img)
         x2d = numpy.zeros_like(y) + x
         y2d = numpy.zeros_like(x) + y
         res1 = b.map_coordinates((y2d, x2d))
-        self.assertEquals(abs(res1 - img).max(), 0, "images are the same (corners)")
+        self.assertEqual(abs(res1 - img).max(), 0, "images are the same (corners)")
 
-        x2d = numpy.zeros_like(y) + (x[:, :-1] + 0.5)
-        y2d = numpy.zeros_like(x[:, :-1]) + y
+        x2d = numpy.zeros_like(y) + (x[:,:-1] + 0.5)
+        y2d = numpy.zeros_like(x[:,:-1]) + y
         res1 = b.map_coordinates((y2d, x2d))
-        self.assertEquals(abs(res1 - img[:, :-1] - 0.5).max(), 0, "images are the same (middle)")
+        self.assertEqual(abs(res1 - img[:,:-1] - 0.5).max(), 0, "images are the same (middle)")
 
-        x2d = numpy.zeros_like(y[:-1, :]) + (x[:, :-1] + 0.5)
-        y2d = numpy.zeros_like(x[:, :-1]) + (y[:-1, :] + 0.5)
+        x2d = numpy.zeros_like(y[:-1,:]) + (x[:,:-1] + 0.5)
+        y2d = numpy.zeros_like(x[:,:-1]) + (y[:-1,:] + 0.5)
         res1 = b.map_coordinates((y2d, x2d))
-        self.assertEquals(abs(res1 - img[:-1, 1:]).max(), 0, "images are the same (center)")
+        self.assertEqual(abs(res1 - img[:-1, 1:]).max(), 0, "images are the same (center)")
+
+    def test_mask_grad(self):
+        N = 100
+        img = numpy.arange(N * N).reshape(N, N)
+        # No mask on the boundaries, makes the test complicated, pixel always separated
+        masked = 2 * numpy.random.randint(0, int((N - 1) / 2), size=(2, N)) + 1
+        mask = numpy.zeros((N, N), dtype=numpy.uint8)
+        mask[(masked[0], masked[1])] = 1
+        self.assertLessEqual(mask.sum(), N, "At most N pixels are masked")
+
+        b = BilinearImage(img, mask=mask)
+        self.assertEqual(b.has_mask, True, "interpolator has mask")
+        self.assertEqual(b.maxi, N * N - 1, "maxi is N²-1")
+        self.assertEqual(b.mini, 0, "mini is 0")
+
+        y, x = numpy.ogrid[:N,:N]
+        x2d = numpy.zeros_like(y) + x
+        y2d = numpy.zeros_like(x) + y
+        res1 = b.map_coordinates((y2d, x2d))
+        self.assertEqual(numpy.nanmax(abs(res1 - img)), 0, "images are the same (corners), or Nan ")
+
+        x2d = numpy.zeros_like(y) + (x[:,:-1] + 0.5)
+        y2d = numpy.zeros_like(x[:,:-1]) + y
+        res1 = b.map_coordinates((y2d, x2d))
+        self.assertLessEqual(numpy.max(abs(res1 - img[:, 1:] + 1 / 2.)), 0.5, "images are the same (middle) +/- 0.5")
+
+        x2d = numpy.zeros_like(y[:-1]) + (x[:,:-1] + 0.5)
+        y2d = numpy.zeros_like(x[:,:-1]) + (y[:-1] + 0.5)
+        res1 = b.map_coordinates((y2d, x2d))
+        exp = 0.25 * (img[:-1,:-1] + img[:-1, 1:] + img[1:,:-1] + img[1:, 1:])
+        self.assertLessEqual(abs(res1 - exp).max(), N / 4, "images are almost the same (center)")
 
     def test_profile_grad(self):
         N = 100
@@ -103,7 +138,7 @@ class TestBilinear(unittest.TestCase):
         b = BilinearImage(img)
         res1 = b.profile_line((0, 0), (N - 1, N - 1))
         l = numpy.ceil(numpy.sqrt(2) * N)
-        self.assertEquals(len(res1), l, "Profile has correct length")
+        self.assertEqual(len(res1), l, "Profile has correct length")
         self.assertLess((res1[:-2] - res1[1:-1]).std(), 1e-3, "profile is linear (excluding last point)")
 
     def test_profile_gaus(self):
@@ -114,8 +149,8 @@ class TestBilinear(unittest.TestCase):
         b = BilinearImage(img)
         res_hor = b.profile_line((N // 2, 0), (N // 2, N - 1))
         res_ver = b.profile_line((0, N // 2), (N - 1, N // 2))
-        self.assertEquals(len(res_hor), N, "Profile has correct length")
-        self.assertEquals(len(res_ver), N, "Profile has correct length")
+        self.assertEqual(len(res_hor), N, "Profile has correct length")
+        self.assertEqual(len(res_ver), N, "Profile has correct length")
         self.assertLess(abs(res_hor - g).max(), 1e-5, "correct horizontal profile")
         self.assertLess(abs(res_ver - g).max(), 1e-5, "correct vertical profile")
 
@@ -124,8 +159,8 @@ class TestBilinear(unittest.TestCase):
         res_hor = b.profile_line((N // 2, 0), (N // 2, N - 1), linewidth=3)
         res_ver = b.profile_line((0, N // 2), (N - 1, N // 2), linewidth=3)
 
-        self.assertEquals(len(res_hor), N, "Profile has correct length")
-        self.assertEquals(len(res_ver), N, "Profile has correct length")
+        self.assertEqual(len(res_hor), N, "Profile has correct length")
+        self.assertEqual(len(res_ver), N, "Profile has correct length")
         self.assertLess(abs(res_hor - expected_profile).max(), 1e-5,
                         "correct horizontal profile")
         self.assertLess(abs(res_ver - expected_profile).max(), 1e-5,
@@ -139,4 +174,5 @@ def suite():
     testsuite.addTest(TestBilinear("test_map"))
     testsuite.addTest(TestBilinear("test_profile_grad"))
     testsuite.addTest(TestBilinear("test_profile_gaus"))
+    testsuite.addTest(TestBilinear("test_mask_grad"))
     return testsuite
