@@ -4,7 +4,7 @@
 #    Project: S I L X project
 #             https://github.com/silx-kit/silx
 #
-#    Copyright (C) 2012-2017 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2012-2021 European Synchrotron Radiation Facility, Grenoble, France
 #
 #    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
 #
@@ -34,7 +34,7 @@ __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "2012-2017 European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "16/10/2017"
+__date__ = "30/11/2020"
 __status__ = "stable"
 __all__ = ["ocl", "pyopencl", "mf", "release_cl_buffers", "allocate_cl_buffers",
            "measure_workgroup_size", "kernel_workgroup_size"]
@@ -46,31 +46,35 @@ import numpy
 
 from .utils import get_opencl_code
 
-
 logger = logging.getLogger(__name__)
 
-
 if os.environ.get("SILX_OPENCL") in ["0", "False"]:
-    logger.warning("Use of OpenCL has been disables from environment variable: SILX_OPENCL=0")
+    logger.info("Use of OpenCL has been disabled from environment variable: SILX_OPENCL=0")
     pyopencl = None
 else:
     try:
         import pyopencl
     except ImportError:
-        logger.warning("Unable to import pyOpenCl. Please install it from: http://pypi.python.org/pypi/pyopencl")
+        logger.warning("Unable to import pyOpenCl. Please install it from: https://pypi.org/project/pyopencl")
         pyopencl = None
     else:
-        import pyopencl.array as array
-        mf = pyopencl.mem_flags
+        try:
+            pyopencl.get_platforms()
+        except pyopencl.LogicError:
+            logger.warning("The module pyOpenCL has been imported but can't be used here")
+            pyopencl = None
+        else:
+            import pyopencl.array as array
+            mf = pyopencl.mem_flags
 
 if pyopencl is None:
+
     # Define default mem flags
     class mf(object):
         WRITE_ONLY = 1
         READ_ONLY = 1
         READ_WRITE = 1
 
-
 FLOP_PER_CORE = {"GPU": 64,  # GPU, Fermi at least perform 64 flops per cycle/multicore, G80 were at 24 or 48 ...
                  "CPU": 4,  # CPU, at least intel's have 4 operation per cycle
                  "ACC": 8}  # ACC: the Xeon-phi (MIC) appears to be able to process 8 Flops per hyperthreaded-core
@@ -91,8 +95,8 @@ NVIDIA_FLOP_PER_CORE = {(1, 0): 24,  # Guessed !
                         (6, 0): 128,  # GP100
                         (6, 1): 128,  # GP104
                         (6, 2): 128,  # ?
-                        (7, 0): 256,  # Volta ?
-                        (7, 1): 256,  # Volta ?
+                        (7, 0): 128,  # Volta # measured on Telsa V100
+                        (7, 1): 128,  # Volta ?
                         }
 
 AMD_FLOP_PER_CORE = 160  # Measured on a M7820 10 core, 700MHz 1120GFlops
@@ -102,6 +106,7 @@ class Device(object):
     """
     Simple class that contains the structure of an OpenCL device
     """
+
     def __init__(self, name="None", dtype=None, version=None, driver_version=None,
                  extensions="", memory=None, available=None,
                  cores=None, frequency=None, flop_core=None, idx=0, workgroup=1):
@@ -158,11 +163,17 @@ class Device(object):
                "Available\t:\t%s" % self.available]
         return os.linesep.join(lst)
 
+    def set_unavailable(self):
+        """Use this method to flag a faulty device
+        """
+        self.available = False
+
 
 class Platform(object):
     """
     Simple class that contains the structure of an OpenCL platform
     """
+
     def __init__(self, name="None", vendor="None", version=None, extensions=None, idx=0):
         """
         Class containing all descriptions of a platform and all devices description within that platform.
@@ -214,14 +225,24 @@ class Platform(object):
 def _measure_workgroup_size(device_or_context, fast=False):
     """Mesure the maximal work group size of the given device
 
+    DEPRECATED since not perfectly correct !
+
     :param device_or_context: instance of pyopencl.Device or pyopencl.Context
                     or 2-tuple (platformid,deviceid)
     :param fast: ask the kernel the valid value, don't probe it
     :return: maximum size for the workgroup
     """
     if isinstance(device_or_context, pyopencl.Device):
-        ctx = pyopencl.Context(devices=[device_or_context])
-        device = device_or_context
+        try:
+            ctx = pyopencl.Context(devices=[device_or_context])
+        except pyopencl._cl.LogicError as error:
+            platform = device_or_context.platform
+            platformid = pyopencl.get_platforms().index(platform)
+            deviceid = platform.get_devices().index(device_or_context)
+            ocl.platforms[platformid].devices[deviceid].set_unavailable()
+            raise RuntimeError("Unable to create context on %s/%s: %s" % (platform, device_or_context, error))
+        else:
+            device = device_or_context
     elif isinstance(device_or_context, pyopencl.Context):
         ctx = device_or_context
         device = device_or_context.devices[0]
@@ -241,7 +262,8 @@ def _measure_workgroup_size(device_or_context, fast=False):
     max_valid_wg = 1
     data = numpy.random.random(shape).astype(numpy.float32)
     d_data = pyopencl.array.to_device(queue, data)
-    d_data_1 = pyopencl.array.zeros_like(d_data) + 1
+    d_data_1 = pyopencl.array.empty_like(d_data)
+    d_data_1.fill(numpy.float32(1.0))
 
     program = pyopencl.Program(ctx, get_opencl_code("addition")).build()
     if fast:
@@ -298,17 +320,28 @@ class OpenCL(object):
                 ####################################################
                 extensions = device.extensions
                 if (pypl.vendor == "NVIDIA Corporation") and ('cl_khr_fp64' in extensions):
-                                extensions += ' cl_khr_int64_base_atomics cl_khr_int64_extended_atomics'
+                    extensions += ' cl_khr_int64_base_atomics cl_khr_int64_extended_atomics'
                 try:
                     devtype = pyopencl.device_type.to_string(device.type).upper()
                 except ValueError:
                     # pocl does not describe itself as a CPU !
                     devtype = "CPU"
                 if len(devtype) > 3:
-                    devtype = devtype[:3]
-                if _is_nvidia_gpu(pypl.vendor, devtype) and "compute_capability_major_nv" in dir(device):
-                    comput_cap = device.compute_capability_major_nv, device.compute_capability_minor_nv
-                    flop_core = NVIDIA_FLOP_PER_CORE.get(comput_cap, min(NVIDIA_FLOP_PER_CORE.values()))
+                    if "GPU" in devtype:
+                        devtype = "GPU"
+                    elif "ACC" in devtype:
+                        devtype = "ACC"
+                    elif "CPU" in devtype:
+                        devtype = "CPU"
+                    else:
+                        devtype = devtype[:3]
+                if _is_nvidia_gpu(device.vendor, devtype) and ("compute_capability_major_nv" in dir(device)):
+                    try:
+                        comput_cap = device.compute_capability_major_nv, device.compute_capability_minor_nv
+                    except pyopencl.LogicError:
+                        flop_core = FLOP_PER_CORE["GPU"]
+                    else:
+                        flop_core = NVIDIA_FLOP_PER_CORE.get(comput_cap, FLOP_PER_CORE["GPU"])
                 elif (pypl.vendor == "Advanced Micro Devices, Inc.") and (devtype == "GPU"):
                     flop_core = AMD_FLOP_PER_CORE
                 elif devtype == "CPU":
@@ -317,7 +350,7 @@ class OpenCL(object):
                     flop_core = 1
                 workgroup = device.max_work_group_size
                 if (devtype == "CPU") and (pypl.vendor == "Apple"):
-                    logger.warning("For Apple's OpenCL on CPU: Measuring actual valid max_work_goup_size.")
+                    logger.info("For Apple's OpenCL on CPU: Measuring actual valid max_work_goup_size.")
                     workgroup = _measure_workgroup_size(device, fast=True)
                 if (devtype == "GPU") and os.environ.get("GPU") == "False":
                     # Environment variable to disable GPU devices
@@ -379,6 +412,8 @@ class OpenCL(object):
         best_found = None
         for platformid, platform in enumerate(self.platforms):
             for deviceid, device in enumerate(platform.devices):
+                if not device.available:
+                    continue
                 if (dtype in ["ALL", "DEF"]) or (device.type == dtype):
                     if (memory is None) or (memory <= device.memory):
                         found = True
@@ -400,7 +435,7 @@ class OpenCL(object):
         return None
 
     def create_context(self, devicetype="ALL", useFp64=False, platformid=None,
-                       deviceid=None, cached=True, memory=None):
+                       deviceid=None, cached=True, memory=None, extensions=None):
         """
         Choose a device and initiate a context.
 
@@ -410,13 +445,20 @@ class OpenCL(object):
         E.g.: If Nvidia driver is installed, GPU will succeed but CPU will fail.
               The AMD SDK kit is required for CPU via OpenCL.
         :param devicetype: string in ["cpu","gpu", "all", "acc"]
-        :param useFp64: boolean specifying if double precision will be used
+        :param useFp64: boolean specifying if double precision will be used: deprecated use extensions=["cl_khr_fp64"]
         :param platformid: integer
         :param deviceid: integer
         :param cached: True if we want to cache the context
         :param memory: minimum amount of memory of the device
+        :param extensions: list of extensions to be present
         :return: OpenCL context on the selected device
         """
+        if extensions is None:
+            extensions = []
+        if useFp64:
+            logger.warning("Deprecation: please select your device using the extension name!, i.e. extensions=['cl_khr_fp64']")
+            extensions.append('cl_khr_fp64')
+
         if (platformid is not None) and (deviceid is not None):
             platformid = int(platformid)
             deviceid = int(deviceid)
@@ -425,20 +467,24 @@ class OpenCL(object):
             pyopencl_ctx += [0] * (2 - len(pyopencl_ctx))  # pad with 0
             platformid, deviceid = pyopencl_ctx
         else:
-            if useFp64:
-                ids = ocl.select_device(type=devicetype, extensions=["cl_khr_int64_base_atomics"])
-            else:
-                ids = ocl.select_device(dtype=devicetype)
+            ids = ocl.select_device(type=devicetype, extensions=extensions)
             if ids:
                 platformid, deviceid = ids
+        ctx = None
         if (platformid is not None) and (deviceid is not None):
             if (platformid, deviceid) in self.context_cache:
                 ctx = self.context_cache[(platformid, deviceid)]
             else:
-                ctx = pyopencl.Context(devices=[pyopencl.get_platforms()[platformid].get_devices()[deviceid]])
-                if cached:
-                    self.context_cache[(platformid, deviceid)] = ctx
-        else:
+                try:
+                    ctx = pyopencl.Context(devices=[pyopencl.get_platforms()[platformid].get_devices()[deviceid]])
+                except pyopencl._cl.LogicError as error:
+                    self.platforms[platformid].devices[deviceid].set_unavailable()
+                    logger.warning("Unable to create context on %s/%s: %s", platformid, deviceid, error)
+                    ctx = None
+                else:
+                    if cached:
+                        self.context_cache[(platformid, deviceid)] = ctx
+        if ctx is None:
             logger.warning("Last chance to get an OpenCL device ... probably not the one requested")
             ctx = pyopencl.create_some_context(interactive=False)
         return ctx
@@ -529,6 +575,53 @@ def allocate_cl_buffers(buffers, device=None, context=None):
     return mem
 
 
+def allocate_texture(ctx, shape, hostbuf=None, support_1D=False):
+    """
+    Allocate an OpenCL image ("texture").
+
+    :param ctx: OpenCL context
+    :param shape: Shape of the image. Note that pyopencl and OpenCL < 1.2
+        do not support 1D images, so 1D images are handled as 2D with one row
+    :param support_1D: force the image to be 1D if the shape has only one dim
+    """
+    if len(shape) == 1 and not(support_1D):
+        shape = (1,) + shape
+    return pyopencl.Image(
+        ctx,
+        pyopencl.mem_flags.READ_ONLY | pyopencl.mem_flags.USE_HOST_PTR,
+        pyopencl.ImageFormat(
+            pyopencl.channel_order.INTENSITY,
+            pyopencl.channel_type.FLOAT
+        ),
+        hostbuf=numpy.zeros(shape[::-1], dtype=numpy.float32)
+    )
+
+
+def check_textures_availability(ctx):
+    """
+    Check whether textures are supported on the current OpenCL context.
+
+    :param ctx: OpenCL context
+    """
+    try:
+        dummy_texture = allocate_texture(ctx, (16, 16))
+        # Need to further access some attributes (pocl)
+        dummy_height = dummy_texture.height
+        textures_available = True
+        del dummy_texture, dummy_height
+    except (pyopencl.RuntimeError, pyopencl.LogicError):
+        textures_available = False
+    # Nvidia Fermi GPUs (compute capability 2.X) do not support opencl read_imagef
+    # There is no way to detect this until a kernel is compiled
+    try:
+        cc = ctx.devices[0].compute_capability_major_nv
+        textures_available &= (cc >= 3)
+    except (pyopencl.LogicError, AttributeError):  # probably not a Nvidia GPU
+        pass
+    #
+    return textures_available
+
+
 def measure_workgroup_size(device):
     """Measure the actual size of the workgroup
 
@@ -555,12 +648,25 @@ def measure_workgroup_size(device):
     return res
 
 
-def kernel_workgroup_size(program, kernel):
-    """Extract the compile time maximum workgroup size
+def query_kernel_info(program, kernel, what="WORK_GROUP_SIZE"):
+    """Extract the compile time information from a kernel
 
     :param program: OpenCL program
     :param kernel: kernel or name of the kernel
-    :return: the maximum acceptable workgroup size for the given kernel
+    :param what: what is the query about ?
+    :return: int or 3-int for the workgroup size.
+    
+    Possible information available are:
+    * 'COMPILE_WORK_GROUP_SIZE': Returns the work-group size specified inside the kernel (__attribute__((reqd_work_gr oup_size(X, Y, Z))))
+    * 'GLOBAL_WORK_SIZE': maximum global size that can be used to execute a kernel  #OCL2.1!
+    * 'LOCAL_MEM_SIZE': amount of local memory in bytes being used by the kernel
+    * 'PREFERRED_WORK_GROUP_SIZE_MULTIPLE': preferred multiple of workgroup size for launch. This is a performance hint.
+    * 'PRIVATE_MEM_SIZE' Returns the minimum amount of private memory, in bytes, used by each workitem in the kernel
+    * 'WORK_GROUP_SIZE': maximum work-group size that can be used to execute a kernel on a specific device given by device
+    
+    Further information on:
+    https://www.khronos.org/registry/OpenCL/sdk/1.1/docs/man/xhtml/clGetKernelWorkGroupInfo.html
+    
     """
     assert isinstance(program, pyopencl.Program)
     if not isinstance(kernel, pyopencl.Kernel):
@@ -569,5 +675,15 @@ def kernel_workgroup_size(program, kernel):
         kernel = program.__getattr__(kernel_name)
 
     device = program.devices[0]
-    query_wg = pyopencl.kernel_work_group_info.WORK_GROUP_SIZE
+    query_wg = getattr(pyopencl.kernel_work_group_info, what)
     return kernel.get_work_group_info(query_wg, device)
+
+
+def kernel_workgroup_size(program, kernel):
+    """Extract the compile time maximum workgroup size
+
+    :param program: OpenCL program
+    :param kernel: kernel or name of the kernel
+    :return: the maximum acceptable workgroup size for the given kernel
+    """
+    return query_kernel_info(program, kernel, what="WORK_GROUP_SIZE")
