@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2015-2017 European Synchrotron Radiation Facility
+# Copyright (c) 2015-2019 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -22,31 +22,48 @@
 # THE SOFTWARE.
 #
 # ###########################################################################*/
-"""Full silx test suite.
-
+"""This package provides access to the full silx test suite.
 
 It is possible to disable tests depending on Qt by setting
-:envvar:`WITH_QT_TEST` environment variable to 'False'.
+`silx.test.utils.test_options.WITH_QT_TEST = False`
 It will skip all tests from :mod:`silx.test.gui`.
 """
 
 __authors__ = ["T. Vincent"]
 __license__ = "MIT"
-__date__ = "22/06/2017"
+__date__ = "21/12/2018"
 
 
 import logging
-import os
 import unittest
 
+from silx.test.utils import test_options
+
 
 logger = logging.getLogger(__name__)
 
 
 def suite():
+    # In case Qt tests are not run, do not load sx as it loads Qt
+    # instead add a skipped test class to the suite
+    if not test_options.WITH_QT_TEST:
+        # Explicitly disabled tests
+        msg = "silx.sx tests disabled %s" % test_options.WITH_QT_TEST_REASON
+        logger.warning(msg)
+
+        class SkipSXTest(unittest.TestCase):
+            def runTest(self):
+                self.skipTest(test_options.WITH_QT_TEST_REASON)
+
+        def test_sx_suite():
+            suite = unittest.TestSuite()
+            suite.addTest(SkipSXTest())
+            return suite
+    else:
+        from .test_sx import suite as test_sx_suite
+
     from . import test_version
     from . import test_resources
-    from . import test_sx
     from ..io import test as test_io
     from ..math import test as test_math
     from ..image import test as test_image
@@ -54,9 +71,10 @@ def suite():
     from ..utils import test as test_utils
     from ..opencl import test as test_ocl
     from ..app import test as test_app
+
     test_suite = unittest.TestSuite()
     # test sx first cause qui tests load ipython module
-    test_suite.addTest(test_sx.suite())
+    test_suite.addTest(test_sx_suite())
     test_suite.addTest(test_gui.suite())
     # then test no-gui tests
     test_suite.addTest(test_utils.suite())
@@ -71,9 +89,13 @@ def suite():
     return test_suite
 
 
-def run_tests():
-    """Run test complete test_suite"""
-    runner = unittest.TextTestRunner()
+def run_tests(*args, **kwargs):
+    """Run test complete test_suite
+
+    Provided arguments are passed to :class:`unittest.TextTestRunner`.
+    """
+    test_options.configure()
+    runner = unittest.TextTestRunner(*args, **kwargs)
     if not runner.run(suite()).wasSuccessful():
         print("Test suite failed")
         return 1
