@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2016 European Synchrotron Radiation Facility
+# Copyright (c) 2016-2019 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 # ###########################################################################*/
 __authors__ = ["T. Vincent", "P. Knobel"]
 __license__ = "MIT"
-__date__ = "02/10/2017"
+__date__ = "08/03/2019"
 
 
 import unittest
@@ -34,6 +34,12 @@ from . import test_array_like
 from . import test_launcher
 from . import test_deprecation
 from . import test_proxy
+from . import test_debug
+from . import test_number
+from . import test_external_resources
+from . import test_enum
+from . import test_testutils
+from . import test_retry
 
 
 def suite():
@@ -44,4 +50,10 @@ def suite():
     test_suite.addTest(test_launcher.suite())
     test_suite.addTest(test_deprecation.suite())
     test_suite.addTest(test_proxy.suite())
+    test_suite.addTest(test_debug.suite())
+    test_suite.addTest(test_number.suite())
+    test_suite.addTest(test_external_resources.suite())
+    test_suite.addTest(test_enum.suite())
+    test_suite.addTest(test_testutils.suite())
+    test_suite.addTest(test_retry.suite())
     return test_suite
